﻿using UnityEngine;
using UnityEngine.UI;
using HarmonyLib;
using System.Collections.Generic;
using System;
using System.IO;

public class TreeDropsModifier : Mod
{
    private Harmony harmony;
    static bool saving = false;
    static string globalFile = Path.Combine(SaveAndLoad.WorldPath, "TreeDropsModifier.json");
    static string fileName
    {
        get
        {
            string time = "";
            if (saving)
                time = DateTime.Now.ToString(SaveAndLoad.dateTimeFormattingSaveFile);
            else
                time = new DateTime(SaveAndLoad.WorldToLoad.lastPlayedDateTicks).ToString(SaveAndLoad.dateTimeFormattingSaveFile);
            string path = Path.Combine(SaveAndLoad.WorldPath, SaveAndLoad.CurrentGameFileName, time);
            if (Directory.Exists(path))
                return Path.Combine(path, "TreeDropsModifier.json");
            return Path.Combine(SaveAndLoad.WorldPath, SaveAndLoad.CurrentGameFileName, time + SaveAndLoad.latestStringNameEnding, "TreeDropsModifier.json");
        }
    }
    public static ItemMultiplier worldMulti = new ItemMultiplier();
    public static ItemMultiplier globalMulti = new ItemMultiplier(getGlobalSaveJson());

    public void Start()
    {
        harmony = new Harmony("com.aidanamite.TreeDropsModifier");
        harmony.PatchAll();
        Debug.Log("Mod Tree Drops Modifier has been loaded!");
    }

    public void OnModUnload()
    {
        harmony.UnpatchAll();
        Debug.Log("Mod Tree Drops Modifier has been unloaded!");
    }

    override public void WorldEvent_WorldSaved()
    {
        saving = true;
        saveJson(worldMulti.toJson());
        saving = false;
    }

    public override void WorldEvent_WorldLoaded()
    {
        worldMulti = new ItemMultiplier(getSaveJson());
    }

    [ConsoleCommand(name: "treemulti", docs: "Syntax: 'treemulti <global|world> <category> [value]'")]
    public static string MyCommand(string[] args)
    {
        if (args.Length < 2)
            return "Not enough arguments";
        if (args.Length > 3)
            return "Too many arguments";
        args[0] = args[0].ToLower();
        args[1] = args[1].ToLower();
        int saveLoc = args[0] == "global" ? 0 : args[0] == "world" ? 1 : -1;
        if (saveLoc == -1)
            return args[0] + " is not \"global\" or \"world\"";
        if (args.Length == 3)
            try
            {
                setMultiplier(saveLoc, args[1], float.Parse(args[2]));
                return ToTitle(args[1]) + " drops are now " + getMultiplier(args[1]);
            }
            catch
            {
                return "Failed to parse \"" + args[2] + "\" as a number";
            }
        return ToTitle(args[1]) + " drops are " + globalMulti.getMultiplier(args[1]);
    }

    static void setMultiplier(int loc, string category,float value)
    {
        if (loc == 1)
        {
            worldMulti.setMultiplier(category, value);
            return;
        }
        globalMulti.setMultiplier(category, value);
        saveGlobalJson(globalMulti.toJson());
    }

    public static float getMultiplier(string category)
    {
        return worldMulti.getMultiplier(category) * globalMulti.getMultiplier(category);
    }

    private static JSONObject getSaveJson()
    {
        JSONObject data;
        try
        {
            data = new JSONObject(File.ReadAllText(fileName));
        }
        catch
        {
            data = JSONObject.Create();
            saveJson(data);
        }
        return data;
    }

    private static void saveJson(JSONObject data)
    {
        try
        {
            File.WriteAllText(fileName, data.ToString());
        }
        catch (Exception err)
        {
            Debug.Log("An error occured while trying to save Tree Drops Modifier settings: " + err.Message);
        }
    }

    private static JSONObject getGlobalSaveJson()
    {
        JSONObject data;
        try
        {
            data = new JSONObject(File.ReadAllText(globalFile));
        }
        catch
        {
            data = JSONObject.Create();
            saveGlobalJson(data);
        }
        return data;
    }

    private static void saveGlobalJson(JSONObject data)
    {
        try
        {
            File.WriteAllText(globalFile, data.ToString());
        }
        catch (Exception err)
        {
            Debug.Log("An error occured while trying to save global Tree Drops Modifier settings: " + err.Message);
        }
    }

    private static string ToTitle(string str)
    {
        if (str.Length == 0)
            return "";
        if (str.Length == 1)
            return str.ToUpper();
        return str.Substring(0, 1).ToUpper() + str.Substring(1).ToLower();
    }
}

[HarmonyPatch(typeof(Inventory), "AddItem", new Type[] { typeof(string) , typeof(int) })]
public class Patch_AddItemToPlayer
{
    static void Prefix(ref string uniqueItemName, ref int amount)
    {
        if (Environment.StackTrace.Contains("at HarvestableTree"))
        {
            amount = (int)Mathf.Round(amount * TreeDropsModifier.getMultiplier(uniqueItemName.ToLower()));
        }
    }
}

public class ItemMultiplier
{
    private List<float> values;
    private List<string> names;
    public ItemMultiplier()
    {
        values = new List<float>();
        names = new List<string>();
        values.Add(1);
        names.Add("all");
    }
    public ItemMultiplier(JSONObject source)
    {
        values = new List<float>();
        names = new List<string>();
        values.Add(1);
        names.Add("all");
        if (!source.IsNull)
        {
            foreach (string item in source.keys)
            {
                float fetchedValue = 1;
                source.GetField(ref fetchedValue, item);
                setMultiplier(item, fetchedValue);
            }
        }
    }
    public float getMultiplier(string category)
    {
        int ind = names.IndexOf(category);
        float rtn = 1;
        if (ind > 0)
            rtn *= values[ind];
        return rtn * values[0];
    }
    public void setMultiplier(string category, float value)
    {
        int ind = names.IndexOf(category);
        if (ind == -1)
        {
            if (value != 1)
            {
                names.Add(category);
                values.Add(value);
            }
            return;
        }
        if (value == 1 && ind != 0)
        {
            names.RemoveAt(ind);
            values.RemoveAt(ind);
        }
        else
        {
            values[ind] = value;
        }
    }
    public void removeMultiplier(string category)
    {
        setMultiplier(category, 1);
    }

    public JSONObject toJson()
    {
        JSONObject main = new JSONObject();
        for (int i = 0; i < names.Count; i++)
            main.AddField(names[i], values[i]);
        return main;
    }
}